@@ -2,15 +2,16 @@ import os
 import re
 from abc import ABC
 from enum import Enum
-from functools import lru_cache
 from math import floor, ceil
+from typing import Type, cast, List
+
 from pydantic import BaseModel, Field
+from pymultirole_plugins.util import timed_lru_cache
 from pymultirole_plugins.v1.formatter import FormatterParameters, FormatterBase
 from pymultirole_plugins.v1.processor import ProcessorBase, ProcessorParameters
-from pymultirole_plugins.v1.schema import Document
+from pymultirole_plugins.v1.schema import Document, AltText
 from starlette.responses import Response, PlainTextResponse
 from transformers import pipeline, SummarizationPipeline, AutoTokenizer
-from typing import Type, cast, List
 
 _home = os.path.expanduser("~")
 xdg_cache_home = os.environ.get("XDG_CACHE_HOME") or os.path.join(_home, ".cache")
@@ -31,9 +32,9 @@ class TrfModel(str, Enum):
 
 
 class SummarizerParameters(FormatterParameters):
-    as_metadata: str = Field(
+    as_altText: str = Field(
         None,
-        description="""If defined generate the summary as a metadata of the input document,
+        description="""If defined generate the summary as an alternative text of the input document,
     if not replace the text of the input document.""",
     )
     model: TrfModel = Field(
@@ -51,13 +52,15 @@ class SummarizerParameters(FormatterParameters):
         0.1,
         description="""Minimum number of tokens of the summary:<br/>
         <li>If int, then consider min_length as the minimum number.
-        <li>If float in the range [0.0, 1.0], then consider min_length as a percentage of the original text length in tokens.""",
+        <li>If float in the range [0.0, 1.0], then consider min_length as a percentage
+         of the original text length in tokens.""",
     )
     max_length: float = Field(
         0.25,
         description="""Maximum number of tokens of the summary:<br/>
         <li>If int, then consider max_length as the maximum number.
-        <li>If float in the range [0.0, 1.0], then consider max_length as a percentage of the original text length in tokens.""",
+        <li>If float in the range [0.0, 1.0], then consider max_length as a percentage
+         of the original text length in tokens.""",
     )
 
     do_sample: bool = Field(
@@ -67,7 +70,10 @@ class SummarizerParameters(FormatterParameters):
     num_beams: int = Field(
         1,
         description="""Number of beams for beam search. 1 means no beam search.br/>
-    Specifying this parameter will lead the model to use beam search instead of greedy search, setting num_beams to 4, will allow the model to lookahead for four possible words (1 in the case of greedy search), to keep the most likely 4 of hypotheses at each time step, and choosing the one that has the overall highest probability.""",
+    Specifying this parameter will lead the model to use beam search instead of greedy search, setting num_beams to 4,
+    will allow the model to lookahead for four possible words (1 in the case of greedy search),
+    to keep the most likely 4 of hypotheses at each time step,
+    and choosing the one that has the overall highest probability.""",
     )
     length_penalty: float = Field(
         1.0,
@@ -78,6 +84,28 @@ class SummarizerParameters(FormatterParameters):
     no_repeat_ngram_size: int = Field(
         0, description="If set to int > 0, all ngrams of that size can only occur once."
     )
+    early_stopping: bool = Field(
+        False,
+        description="""Whether to stop the beam search when at least `num_beams`
+        sentences are finished per batch or not.""",
+    )
+    temperature: float = Field(
+        1.0,
+        description="""The value used to module the next token probabilities.""",
+    )
+    top_k: int = Field(
+        50,
+        description="""The number of highest probability vocabulary tokens to keep for top-k-filtering.""",
+    )
+    top_p: float = Field(
+        1.0,
+        description="""If set to float < 1, only the most probable tokens with probabilities that add up to `top_p` or higher
+                are kept for generation.""",
+    )
+    num_return_sequences: int = Field(
+        1,
+        description="""The number of independently computed returned sequences for each element in the batch.""",
+    )
 
 
 def WHITESPACE_HANDLER(text):
@@ -108,10 +136,9 @@ class SummarizerFormatter(FormatterBase, ProcessorBase, ABC):
         try:
             model_max_length = (
                 p.tokenizer.model_max_length
-                if (
-                    p.tokenizer.model_max_length
+                if (p.tokenizer.model_max_length
                     and p.tokenizer.model_max_length < MAX_LENGTH_BUG
-                )
+                    )
                 else 512
             )
             inputs = p.tokenizer(
@@ -146,6 +173,11 @@ class SummarizerFormatter(FormatterBase, ProcessorBase, ABC):
                 do_sample=params.do_sample,
                 length_penalty=params.length_penalty,
                 no_repeat_ngram_size=params.no_repeat_ngram_size,
+                early_stopping=params.early_stopping,
+                temperature=params.temperature,
+                top_k=params.top_k,
+                top_p=params.top_p,
+                num_return_sequences=params.num_return_sequences
             )
             summary = p.tokenizer.decode(output[0], skip_special_tokens=True)
         except Exception as e:
@@ -158,15 +190,18 @@ class SummarizerFormatter(FormatterBase, ProcessorBase, ABC):
         return PlainTextResponse(summary)
 
     def process(
-        self, documents: List[Document], parameters: ProcessorParameters
+            self, documents: List[Document], parameters: ProcessorParameters
     ) -> List[Document]:
         params: SummarizerParameters = cast(SummarizerParameters, parameters)
         for document in documents:
             summary = self._summarize(document, parameters)
-            if params.as_metadata is not None and len(params.as_metadata):
-                if not document.metadata:
-                    document.metadata = {}
-                document.metadata[params.as_metadata] = summary
+            if params.as_altText is not None and len(params.as_altText):
+                document.altTexts = document.altTexts or []
+                altTexts = [
+                    alt for alt in document.altTexts if alt.name != params.as_altText
+                ]
+                altTexts.append(AltText(name=params.as_altText, text=summary))
+                document.altTexts = altTexts
             else:
                 document.text = summary
                 document.annotations = None
@@ -178,7 +213,8 @@ class SummarizerFormatter(FormatterBase, ProcessorBase, ABC):
         return SummarizerParameters
 
 
-@lru_cache(maxsize=None)
+# Huggingface models are dropped after 1 hour
+@timed_lru_cache(seconds=3600, maxsize=None)
 def get_pipeline(model):
     p = pipeline(
         "summarization",
